#ifndef NICVECTOR_H_UNIVERSITY_OF_OREGON_NIC
#define NICVECTOR_H_UNIVERSITY_OF_OREGON_NIC

#include <iostream>

// yet another refactoring and incarnation of the nefarious
// NICVector class. It might make sense to try to replace the
// HiPerSAT implementation with this one, which is arguably 
// cleaner

template <class T> class NICMatrix;
template <class T> class NICVector;

template <class T>
std::ostream& operator<<( std::ostream& out, const NICVector<T>& vec );
template <class T>
std::istream& operator>>( std::istream& is, NICVector<T>& vec );

template <class T>
class NICVector 
{
public:
    enum FileMode
    {
        BINARY = 1,
        ASCII = 2,
        LITTLEENDIAN = 4,
        BIGENDIAN = 8,
        NATIVE = 16
    };

    NICVector();
    NICVector( int size );
    NICVector( const NICVector<T>& vec );
    ~NICVector();

    // vector initialization functions
    void zeroVector();
    void constantVector( const T& v );
    void randomVector(double min = 0, double max = 1);
    void normalize();

    // vector operatons
    void swapVectors( NICVector<T>& vec );
    NICVector<T>& operator=( const NICVector<T>& vec );

    // comparison
    bool operator==( const NICVector<T>& vec ) const;

    //  matrix operations
    void orthogonalize( NICMatrix<T>& mat );

    // input/output functions
    friend std::ostream& operator<< <>( std::ostream& out, const NICVector<T>& v );
    friend std::istream& operator>> <>( std::istream& is, NICVector<T>& v );
    bool read( std::istream& is, FileMode mode = (BINARY | NATIVE) );
    bool read( 
        std::istream& is, int numberToRead, FileMode mode = (BINARY | NATIVE) );
    bool write( std::ostream& os, FileMode mode = (BINARY | NATIVE) );
    bool write( 
        std::ostream& os, int numberToWrite, FileMode mode = (BINARY | NATIVE) );

    // access methods
    T* data();

    // setting the data to an external data source
    void data( T* data, int size );
    int size() const;


private:

    int m_size;
    T* m_data;

    // if we're using an external pointer to the data, we don't want to deallocate
    // it and cause memory management problems for the rest of the program
    bool m_noClobber;

    bool checkFileBits( FileMode mode );
};


#endif
// NICVECTOR_H_UNIVERSITY_OF_OREGON_NIC
